/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

  As a special exception, permission is given to link this program
  with any edition of Qt, and distribute the resulting executable,
  without including the source code for Qt in the source distribution.
*/

#include "MabumbiSysTray.h"

#include <KDE/KAction>
#include <KDE/KActionCollection>
#include <KDE/KIconLoader>

#include <QDebug>
#include <QMenu>

MabumbiSysTray::MabumbiSysTray( QWidget* parent, KActionCollection* mainCollection )
        : KSystemTrayIcon(parent),
        animationIdx(0)
{
    buildContextMenu( mainCollection );
    animationTimer.setInterval( 600 );
    animationTimer.setSingleShot( false );
    connect( &animationTimer, SIGNAL(timeout()), this, SLOT(animate()) );

    setIcon( idleIcon() );
}

MabumbiSysTray::~MabumbiSysTray()
{
}

QIcon MabumbiSysTray::runningIcon()
{
    if ( runIcons.isEmpty() )
    {
        loadRunningIcons();
        if ( runIcons.isEmpty() )
            return QIcon();
    }
    return runIcons[0];
}

void MabumbiSysTray::loadRunningIcons()
{
    QPixmap pix = KIconLoader::global()->loadIcon( "mabumbi-running01", KIconLoader::NoGroup, 32 );
    runIcons.append( QIcon(pix) );
    pix = KIconLoader::global()->loadIcon( "mabumbi-running02", KIconLoader::NoGroup, 32 );
    runIcons.append( QIcon(pix) );
    pix = KIconLoader::global()->loadIcon( "mabumbi-running03", KIconLoader::NoGroup, 32 );
    runIcons.append( QIcon(pix) );
}

QIcon MabumbiSysTray::idleIcon()
{
    QPixmap pix = KIconLoader::global()->loadIcon( "mabumbi-isidle", KIconLoader::NoGroup, 32 );
    return QIcon( pix );
}

QIcon MabumbiSysTray::disconnectedIcon()
{
    QPixmap pix = KIconLoader::global()->loadIcon( "mabumbi-disconnected", KIconLoader::NoGroup, 32 );
    return QIcon( pix );
}

QIcon MabumbiSysTray::errorIcon()
{
    QPixmap pix = KIconLoader::global()->loadIcon( "mabumbi-error", KIconLoader::NoGroup, 32 );
    return QIcon( pix );
}

void MabumbiSysTray::backupRunning()
{
    setIcon( runningIcon() );
    animationTimer.start();
    setToolTip( "Backup is running" );
}

void MabumbiSysTray::backupIdle()
{
    animationTimer.stop();
    setIcon( idleIcon() );
    setToolTip( "Backup is idle" );
}

void MabumbiSysTray::destinationConnected()
{
    backupIdle();
}

void MabumbiSysTray::destinationDisconnected()
{
    animationTimer.stop();
    setIcon( disconnectedIcon() );
    setToolTip( "Backup destination is not available" );
}

void MabumbiSysTray::backupError()
{
    animationTimer.stop();
    setIcon( errorIcon() );
    setToolTip( "Backup error" );
}

void MabumbiSysTray::buildContextMenu( KActionCollection* mainCollection )
{
    QMenu* contextM = contextMenu();
    QAction* actionStart = mainCollection->action( "Start_Backup" );
    if ( actionStart != 0 )
        contextM->addAction( actionStart );
    QAction* actionOpen = mainCollection->action( "Open_Backup" );
    if ( actionOpen != 0 )
        contextM->addAction( actionOpen );

    QAction* titleAction = contextMenuTitle();
    titleAction->setText( "Mabumbi Backup" );
    setContextMenuTitle( titleAction );
}

void MabumbiSysTray::animate()
{
    if ( runIcons.isEmpty() )
    {
        animationTimer.stop();
        return;
    }
    ++animationIdx;
    animationIdx = animationIdx % runIcons.size();
    setIcon( runIcons[animationIdx] );
}

#include "MabumbiSysTray.moc"
